# data and code for multiperiod portfolio rebalancing problem
import numpy as np
import time

ANSWERS = []
TIME = 0
T = 100
n = 5
gamma = 8.0
threshold = 0.001
Sigma = np.array([[  1.512e-02,  1.249e-03,  2.762e-04, -5.333e-03, -7.938e-04],
 [  1.249e-03,  1.030e-02,  6.740e-05, -1.301e-03, -1.937e-04],
 [  2.762e-04,  6.740e-05,  1.001e-02, -2.877e-04, -4.283e-05],
 [ -5.333e-03, -1.301e-03, -2.877e-04,  1.556e-02,  8.271e-04],
 [ -7.938e-04, -1.937e-04, -4.283e-05,  8.271e-04,  1.012e-02]])
mug = np.asarray([ 1.02 , 1.028, 1.01 , 1.034, 1.017])
mu = np.asmatrix([ 1.02 , 1.028, 1.01 , 1.034, 1.017])
kappa_1 = np.matrix([ 0.002, 0.002, 0.002, 0.002, 0.002])
kappa_2 = np.matrix([ 0.004, 0.004, 0.004, 0.004, 0.004])

## Generate returns
# call this function to generate a vector r of market returns
generateReturns = lambda: np.random.multivariate_normal(mug,Sigma)


from cvxpy import *
import copy

#Getting w_star
kappa = kappa_2

w = Variable(n)
obj = Maximize(mu * w  - gamma/2 * quad_form(w,Sigma))
constraints = [sum(w) == 1]
prob = Problem(obj, constraints)
tic = time.time()
ANSWERS.append(prob.solve())
toc = time.time()
TIME += toc - tic
w_star = w.value
ws = []
us = []

w_last = w_star
wt = Parameter(n)
w = Variable(n)
obj = Maximize(mu * w  - gamma/2 * quad_form(w,Sigma) - kappa * abs(w - wt)) 
constraints = [sum(w) == 1]
problem = Problem(obj, constraints)
for i in range(T):
	returns = generateReturns()
	wt.value = np.diag(returns) * w_last / ( returns.T *  w_last )
	EPS=1e-4
	tic = time.time()
	ANSWERS.append(problem.solve(solver="ECOS"))
	toc = time.time()
	TIME += toc - tic
	us.append(w.value - wt.value)
	w_last = w.value
	ws.append(w_last)


ws = np.asarray(ws)
us = np.asarray(us)
w_star = np.asarray(w_star)

ws = ws[:,:,0]
us = us[:,:,0]
w_star = w_star[:,0]



## Plotting code
# You must provide three objects:
# - ws: np.array of size T x n,
#       the post-trade weights w_t_tilde;
# - us: np.array of size T x n, 
#       the trades at each period: w_t_tilde - w_t;
# - w_star: np.array of size n,
#       the "target" solution w_star.
import matplotlib.pyplot as plt
colors = ['b','r','g','c','m']
plt.figure(figsize=(13,5))
for j in range(n):
    plt.plot(range(T), ws[:,j], colors[j])
    plt.plot(range(T), [w_star[j]]*T,  colors[j]+'--')
    non_zero_trades = abs(us[:,j]) > threshold
    print non_zero_trades
plt.ylabel('post-trade weights')
plt.xlabel('period $t$')

num_non_zero_trades = 0.0
for i in range(len(us)):
	moved = [us[i,j] > 1e-3 for j in range(len(us[0]))]
	if any(moved):
		num_non_zero_trades +=1

print num_non_zero_trades/(T)

plt.show()




# coding: utf-8

# These are a list of problems where parsing is slow in CVXPY.
# 
# The first four are all from the Convex.jl paper.
# 
# Be careful when profiling CVXPY that you create a new problem each time.
# When you solve a problem it caches the cone program matrices, so if you solve it again it doesn't run the matrix stuffing algorithm.

# In[9]:

# Preparation:
from cvxpy import *
import numpy


# In[10]:

# Summation.
n = 10000
x = Variable()
e = 0
for i in range(n):
    e = e + x
p = Problem(Minimize(norm(e-1,2)), [x>=0])


# In[11]:

# baseline cvxpy
settings.USE_CVXCANON = False


# In[12]:

get_ipython().run_cell_magic(u'timeit', u'', u"# This generates the cone program matrices but doesn't solve the cone program.\n# 99% of the time here is spent in the matrix stuffing algorithm (going from LinOps to the final matrix).\np = Problem(Minimize(norm(e-1,2)), [x>=0])\np.get_problem_data(ECOS)")


# In[13]:

# CVXPY backed with CVXCanon
settings.USE_CVXCANON = True


# In[15]:

get_ipython().run_cell_magic(u'timeit', u'', u'p = Problem(Minimize(norm(e-1,2)), [x>=0])\np.get_problem_data(ECOS)')


# You can replace %%timeit with %%prun to profile the script.

# In[16]:

# Indexing.
n = 10000
x = Variable(n)
e = 0
for i in range(n):
    e += x[i];


# In[17]:

# baseline cvxpy
settings.USE_CVXCANON = False


# In[18]:

get_ipython().run_cell_magic(u'timeit', u'', u'p = Problem(Minimize(norm(e-1,2)), [x>=0])\np.get_problem_data(ECOS)')


# In[19]:

# CVXPY backed with CVXCanon
settings.USE_CVXCANON = True


# In[34]:

get_ipython().run_cell_magic(u'timeit', u'', u'p = Problem(Minimize(norm(e-1,2)), [x>=0])\np.get_problem_data(ECOS)')


# In[35]:

# Transpose
n = 500
A = numpy.random.randn(n,n)


# In[36]:

# baseline cvxpy
settings.USE_CVXCANON = False


# In[37]:

get_ipython().run_cell_magic(u'timeit', u'', u"X = Variable(n,n)\np = Problem(Minimize(norm(X.T-A,'fro')), [X[1,1] == 1])\np.get_problem_data(ECOS)")


# In[38]:

# CVXPY backed with CVXCanon
settings.USE_CVXCANON = True


# In[39]:

get_ipython().run_cell_magic(u'timeit', u'', u"X = Variable(n,n)\np = Problem(Minimize(norm(X.T-A,'fro')), [X[1,1] == 1])\np.get_problem_data(ECOS)")


# In[40]:

# Matrix constraint.
# CVXPY actually does a pretty good job with this one.
# Convex.jl and CVX are slower (at least when they were profiled for the paper).
n = 500
A = numpy.random.randn(n,n)
B = numpy.random.randn(n,n)


# In[41]:

# baseline cvxpy
settings.USE_CVXCANON = False


# In[42]:

get_ipython().run_cell_magic(u'timeit', u'', u"X = Variable(n,n)\np = Problem(Minimize(norm(X-A,'fro')), [X == B])\np.get_problem_data(ECOS)")


# In[43]:

# CVXPY backed with CVXCanon
settings.USE_CVXCANON = True


# In[44]:

get_ipython().run_cell_magic(u'timeit', u'', u"X = Variable(n,n)\np = Problem(Minimize(norm(X-A,'fro')), [X == B])\np.get_problem_data(ECOS)")


# In[45]:

# Matrix product.
# This one is a bit different, because the issue is that the coefficient for A.T*X*A has n^4 nonzeros.
# A fix is to introduce the variable A.T*X = Y, and rewrite A.T*X*A as Y*A. 
# This will only add 2n^3 nonzeros.
n = 50
A = numpy.random.randn(n,n)


# In[46]:

# baseline cvxpy
settings.USE_CVXCANON = False


# In[47]:

get_ipython().run_cell_magic(u'timeit', u'', u"X = Variable(n,n)\np = Problem(Minimize(norm(X,'fro')), [A.T*X*A >= 1])\np.get_problem_data(ECOS)")


# In[48]:

# CVXPY backed with CVXCanon
settings.USE_CVXCANON = True


# In[49]:

get_ipython().run_cell_magic(u'timeit', u'', u"X = Variable(n,n)\np = Problem(Minimize(norm(X,'fro')), [A.T*X*A >= 1])\np.get_problem_data(ECOS)")


# In[50]:

# SVM with indexing.
def gen_data(n):
    pos = numpy.random.multivariate_normal([1.0,2.0],numpy.eye(2),size=n)
    neg = numpy.random.multivariate_normal([-1.0,1.0],numpy.eye(2),size=n)
    return pos, neg

N = 2
C = 10
pos, neg = gen_data(500)

w = Variable(N)
b = Variable()
xi_pos = Variable(pos.shape[0])
xi_neg = Variable(neg.shape[0])
cost = sum_squares(w) + C*sum_entries(xi_pos) + C*sum_entries(xi_neg)
constrs = []
for j in range(pos.shape[0]):
    constrs += [w.T*pos[j,:] - b >= 1 - xi_pos[j]]
    
for j in range(neg.shape[0]):
    constrs += [-(w.T*neg[j,:] - b) >= 1 - xi_neg[j]]


# In[51]:

# baseline cvxpy
settings.USE_CVXCANON = False


# In[52]:

get_ipython().run_cell_magic(u'timeit', u'', u'p = Problem(Minimize(cost), constrs)\np.get_problem_data(ECOS)')


# In[53]:

# CVXPY backed with CVXCanon
settings.USE_CVXCANON = True


# In[54]:

get_ipython().run_cell_magic(u'timeit', u'', u'p = Problem(Minimize(cost), constrs)\np.get_problem_data(ECOS)')


# In[55]:




# In[ ]:






import unittest
from cvxpy import *
import numpy as np
from cvxpy.tests.base_test import *
import scipy.sparse as sp


class TestLinOps(BaseTest):
    def assertDataEqual(self, original_data, canon_data):
        for key in ['A', 'b', 'c', 'G', 'h']:
            M1, M2 = original_data[key], canon_data[key]
            if isinstance(M1, sp.csc.csc_matrix):
                self.assertTrue((M1 - M2).nnz == 0)
            else:
                self.assertTrue(np.allclose(M1, M2))

    def assertConstraintsMatch(self, constraints):
        settings.USE_CVXCANON = False
        p = Problem(Minimize(0), constraints)
        cvxpy_data = p.get_problem_data(ECOS)

        settings.USE_CVXCANON = True
        p = Problem(Minimize(0), constraints)
        cvx_canon_data = p.get_problem_data(ECOS)

        self.assertDataEqual(cvxpy_data, cvx_canon_data)

    def test_sum_dense(self):
        rows, cols = 3, 3
        x1 = Variable(rows, cols)
        x2 = Variable(rows, cols)
        A = np.random.randn(rows, cols)
        self.assertConstraintsMatch([x1 + x2 == A])

    def test_sum_sparse(self):
        rows, cols = 75, 100
        x1 = Variable(rows, cols)
        x2 = Variable(rows, cols)
        A_sp = sp.rand(rows, cols, density=0.01)
        self.assertConstraintsMatch([x1 + x2 == A_sp])

    def test_promote(self):
        rows = 10
        b = Variable()
        v1 = Variable(rows, 1)
        self.assertConstraintsMatch([v1 == b])

    def test_mul_dense(self):
        rows, cols = 15, 17

        A = np.random.randn(rows, cols)
        b = Variable(rows, 1)
        x = Variable(cols, 1)
        y = Variable(rows, 1)
        self.assertConstraintsMatch([A*x == b, A*x == y])

    def test_mul_sparse(self):
        rows, cols = 75, 100

        A_sp = Constant(sp.rand(rows, cols, density=0.01))
        b = Variable(rows, 1)
        x = Variable(cols, 1)
        y = Variable(rows, 1)
        self.assertConstraintsMatch([A_sp*x == b, A_sp*x == y])

    def test_rmul_dense(self):
        rows, cols = 15, 17
        A = np.random.randn(rows, cols)
        b = Variable(1, cols)
        x = Variable(1, rows)
        y = Variable(1, cols)
        self.assertConstraintsMatch([x * A == b, x * A == y])

    def test_rmul_sparse(self):
        rows, cols = 75, 100
        A_sp = Constant(sp.rand(rows, cols, density=0.01))
        b = Variable(1, cols)
        x = Variable(1, rows)
        y = Variable(1, cols)
        self.assertConstraintsMatch([x * A_sp == b, x * A_sp == y])

    def test_mul_elemwise_dense(self):
        rows, cols = 15, 17
        A = np.random.randn(rows, cols)
        x1 = Variable(rows, cols)
        M = Variable(rows, cols)
        self.assertConstraintsMatch([mul_elemwise(A, x1) == M])

    def test_mul_elemwise_sparse(self):
        rows, cols = 75, 100
        A_sp = Constant(sp.rand(rows, cols, density=0.01))
        x1 = Variable(rows, cols)
        M = Variable(rows, cols)
        self.assertConstraintsMatch([mul_elemwise(A_sp, x1) == M])

    def test_div(self):
        x = Variable()
        self.assertConstraintsMatch([x / 4 == 3])

    def test_neg_dense(self):
        rows, cols = 3, 3
        x1 = Variable(rows, cols)
        x2 = Variable(rows, cols)
        A = np.random.randn(rows, cols)
        self.assertConstraintsMatch([x1 - x2 == A])

    def test_neg_sparse(self):
        rows, cols = 3, 3
        x1 = Variable(rows, cols)
        x2 = Variable(rows, cols)
        A_sp = sp.rand(rows, cols, density=0.01)
        self.assertConstraintsMatch([x1 - x2 == A_sp])

    def test_index(self):
        rows, cols = 15, 17
        X = Variable(rows, cols)

        for trial in xrange(100):
            xi = np.random.choice(rows)
            xj = np.random.choice(rows)
            xk = np.random.choice(rows)
            ry = np.random.choice(cols)
            if xk == 0:
                xk = 1
            # if np.random.rand() < 0.5:
            #     xk *= -1

            yi = np.random.choice(cols)
            yj = np.random.choice(cols)
            yk = np.random.choice(cols)
            rx = np.random.choice(rows)
            if yk == 0:
                yk = 1
            # if np.random.rand() < 0.5:
            #     yk *= -1

            constr = [X[xi:xj:xk, ry] == 0, X[rx, yi:yj:yk] == 0]
            self.assertConstraintsMatch(constr)

    def test_transpose_dense(self):
        rows, cols = 15, 17
        X = Variable(rows, cols)
        A = np.random.randn(rows, cols)
        self.assertConstraintsMatch([X.T == A.T])

    def test_transpose_sparse(self):
        rows, cols = 75, 100
        X = Variable(rows, cols)
        A_sp = sp.rand(rows, cols, density=0.01)
        self.assertConstraintsMatch([X.T == A_sp.T])

    def test_sum_entries(self):
        rows, cols = 15, 17
        X = Variable(rows, cols)
        self.assertConstraintsMatch([sum_entries(X) == 4.5])

    def test_trace(self):
        n = 15
        X = Variable(n, n)
        self.assertConstraintsMatch([trace(X) == 3])

    def test_reshape_dense(self):
        rows, cols = 15, 17
        X = Variable(rows, cols)

        while True:
            n = np.random.choice(rows + cols)
            if n == 0:
                n = 1
            if rows * cols % n == 0:
                break
        m = (rows * cols) / n

        A = np.random.randn(m, n)

        constr = [reshape(X, m, n) == A]
        self.assertConstraintsMatch(constr)

    def test_reshape_sparse(self):
        rows, cols = 75, 100
        X = Variable(rows, cols)

        while True:
            n = np.random.choice(rows + cols)
            if n == 0:
                n = 1
            if rows * cols % n == 0:
                break
        m = (rows * cols) / n

        A = sp.rand(m, n, density=0.01)

        constr = [reshape(X, m, n) == A]
        self.assertConstraintsMatch(constr)

    def test_diag_vec_dense(self):
        n = 15
        x = Variable(n, 1)
        constr = [diag(x) == np.eye(n)]
        self.assertConstraintsMatch(constr)

    def test_diag_vec_sparse(self):
        n = 15
        x = Variable(n, 1)
        constr = [diag(x) == sp.eye(n)]
        self.assertConstraintsMatch(constr)

    def test_diag_mat_dense(self):
        n = 15
        X = Variable(n, n)
        v = np.random.randn(n)
        constr = [diag(X) == v]
        self.assertConstraintsMatch(constr)

    def test_diag_mat_sparse(self):
        n = 15
        X = Variable(n, n)
        v = sp.rand(n, 1, density=0.01)
        constr = [diag(X) == v]
        self.assertConstraintsMatch(constr)

    def test_upper_tri(self):
        n = 15
        X = Variable(n, n)
        constr = [upper_tri(X) == 0]
        self.assertConstraintsMatch(constr)

    def test_conv(self):
        n, m = 15, 17
        c = np.random.randn(m, 1)
        x = Variable(n, 1)
        y = np.random.randn(n + m - 1, 1)

        constr = [conv(c, x) == y]
        self.assertConstraintsMatch(constr)

    def test_hstack_dense(self):
        rows, cols = 15, 17
        X1 = Variable(rows, cols)
        X2 = Variable(rows, cols)
        X3 = Variable(rows, cols)
        X4 = Variable(rows, cols)
        X5 = Variable(rows, cols)

        A = np.random.randn(rows, cols * 5)
        constr = [hstack(X1, X2, X3, X4, X5) == A]
        self.assertConstraintsMatch(constr)

    def test_hstack_sparse(self):
        rows, cols = 15, 17
        X1 = Variable(rows, cols)
        X2 = Variable(rows, cols)
        X3 = Variable(rows, cols)
        X4 = Variable(rows, cols)
        X5 = Variable(rows, cols)

        A_sp = sp.rand(rows, cols * 5, density=0.01)
        constr = [hstack(X1, X2, X3, X4, X5) == A_sp]
        self.assertConstraintsMatch(constr)

    def test_vstack_dense(self):
        rows, cols = 15, 17
        X1 = Variable(rows, cols)
        X2 = Variable(rows, cols)
        X3 = Variable(rows, cols)
        X4 = Variable(rows, cols)
        X5 = Variable(rows, cols)

        A = np.random.randn(5 * rows, cols)
        constr = [vstack(X1, X2, X3, X4, X5) == A]
        self.assertConstraintsMatch(constr)

    def test_vstack_sparse(self):
        rows, cols = 75, 100
        X1 = Variable(rows, cols)
        X2 = Variable(rows, cols)
        X3 = Variable(rows, cols)
        X4 = Variable(rows, cols)
        X5 = Variable(rows, cols)

        A_sp = sp.rand(5 * rows, cols, density=0.01)
        constr = [vstack(X1, X2, X3, X4, X5) == A_sp]
        self.assertConstraintsMatch(constr)

    def test_kron_dense(self):
        m, n = 3, 5
        p, q = 7, 9
        X = Variable(m, n)
        C = np.random.randn(p, q)
        A = np.random.randn(p * m, q * n)
        constr = [kron(C, X) == A]
        self.assertConstraintsMatch(constr)

    def test_kron_sparse(self):
        m, n = 13, 15
        p, q = 17, 19
        X = Variable(m, n)
        C = sp.rand(p, q, density=0.1)
        A = sp.rand(p * m, q * n, density=0.1)
        constr = [kron(C, X) == A]
        self.assertConstraintsMatch(constr)

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestLinOps)
    unittest.TextTestRunner(verbosity=2).run(suite)



from cvxpy import *
from os import listdir
import numpy
import glob
import os
import subprocess
import time
import numpy as np
import matplotlib.pyplot as plt



def run_testfile(filename):
	print filename


	settings.USE_CVXCANON = False	
	exec("from cvxpy import *\nsettings.USE_CVXCANON=False\n" + open(filename).read())
	oldtime = TIME
	OLD_ANSWERS = ANSWERS


	settings.USE_CVXCANON = True
	exec("from cvxpy import *\nsettings.USE_CVXCANON=True\n" + open(filename).read())
	newtime =  TIME
	NEW_ANSWERS = ANSWERS
	if(len(NEW_ANSWERS) != len(OLD_ANSWERS)):
		print "**** TEST " + filename + " FAILED: Different Number of Answers *****"
	for i in range(min(len(OLD_ANSWERS), len(NEW_ANSWERS))):
		if (OLD_ANSWERS[i] == -float('inf') and NEW_ANSWERS[i] != -float('inf')):
			print "**** TEST " + filename + " FAILED: Different Answers *****"
			return (oldtime, newtime)
		elif(OLD_ANSWERS[i] == float('inf') and NEW_ANSWERS[i] != float('inf')):
			print "**** TEST " + filename + " FAILED: Different Answers *****"
			return (oldtime, newtime)
		elif(OLD_ANSWERS[i] == NEW_ANSWERS[i]):
			continue
		elif(np.abs(OLD_ANSWERS[i] - NEW_ANSWERS[i]) > 1e-6 ):
			print "**** TEST " + filename + " FAILED: Different Answers *****"
			return (oldtime, newtime)
	

	print "***** TEST " + filename + " SUCCESSS, SAME ANSWERS ******"
	return (oldtime, newtime)

files = glob.glob("./364A_scripts/*.py")

oldtime = {};
newtime = {};
fnames = [];
iters = 30

for testfile in files:
	oldtime[testfile] = []
	newtime[testfile] = []
	
	for _ in range(30):
		print "testing: ", testfile
		try:
			(o, n) = run_testfile(testfile);
			oldtime[testfile] += [o]
			newtime[testfile] += [n]
			fnames += [testfile]
		except (RuntimeError, TypeError, NameError, AttributeError):
			pass
list(set(fnames))


import numpy as np
from cvxpy import *
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import copy
import math
import time



# data file for flux balance analysis in systems biology
# From Segre, Zucker et al "From annotated genomes to metabolic flux
# models and kinetic parameter fitting" OMICS 7 (3), 301-316. 
import numpy as np


TIME = 0
# Stoichiometric matrix
#	columns are M1	M2	M3	M4	M5	M6	
# For your interest, the rows correspond to the following equations
#	R1:  extracellular -->  M1
#	R2:  M1 -->  M2
#	R3:  M1 -->  M3
#	R4:  M2 + M5 --> 2 M4
#	R5:  extracellular -->  M5
#	R6:  2 M2 -->  M3 + M6
#	R7:  M3 -->  M4
#	R8:  M6 --> extracellular
#	R9:  M4 --> cell biomass
S = np.matrix("""
	1,0,0,0,0,0;
	-1,1,0,0,0,0;
	-1,0,1,0,0,0;
	0,-1,0,2,-1,0;
	0,0,0,0,1,0;
	0,-2,1,0,0,1;
	0,0,-1,1,0,0;
	0,0,0,0,0,-1;
	0,0,0,-1,0,0
	""").T

m,n = S.shape

vmax = np.matrix("""
	10.10;
	100;
	5.90;
	100;
	3.70;
	100;
	100;
	100;
	100
	""")

v = Variable(n)

ANSWERS = []
constraints = [S*v == 0, v <= vmax, 0<=v]
objective = Maximize(v[-1])
prob = Problem(objective, constraints)

tic = time.time()
val_orig = prob.solve()
toc = time.time()
TIME += toc - tic 

ANSWERS.append(val_orig)
pass #print "Maximal rate: ", val_orig
pass #print "L1: ", constraints[0].dual_value
pass #print "L2: ", constraints[1].dual_value
pass #print "L3: ", constraints[2].dual_value


for i in range(len(vmax)):
	vmax_prime = copy.deepcopy(vmax) 
	vmax_prime[i] = 0
	constraints = [S*v == 0, v <= vmax_prime, 0<=v]
	objective = Maximize(v[-1])
	prob = Problem(objective, constraints)
	
	tic = time.time()
	val = prob.solve()
	toc = time.time()

	TIME += toc - tic
	ANSWERS.append(val)
	if val < .2 * val_orig:
		pass #print "Essential gene:", i	



for i in range(len(vmax)):
	vmax_prime = copy.deepcopy(vmax) 
	for j in range(i+1, len(vmax)):
		vmax_prime[i] = 0
		vmax_prime[j] = 0
		constraints = [S*v == 0, v <= vmax_prime, 0<=v]
		objective = Maximize(v[-1])
		prob = Problem(objective, constraints)
		
		tic = time.time()
		val = prob.solve()
		toc = time.time()

		TIME += toc - tic
		ANSWERS.append(val)
		if val < .2 * val_orig:
			pass #print "Synthetic lethal:", i, j


import numpy as np
from cvxpy import *
import matplotlib.pyplot as pyplot
import time

TIME = 0

Sigma = Semidef(4)
ANSWERS = []
x = np.array([.1, .2, -.05, .1])

constraints = [Sigma == Sigma.T]

constraints += [Sigma[0,1] >= 0, Sigma[0,2] >= 0,\
Sigma[1,2] <= 0, Sigma[2,3] <= 0, Sigma[1,3]<=0,\
Sigma[0,0] == .2, Sigma[1,1] == .1, Sigma[2,2] == .3,\
Sigma[3,3] == .1]


objective = Maximize(quad_form(x,Sigma))

prob = Problem(objective, constraints)
tic = time.time()
risk = prob.solve()
toc = time.time()
TIME += toc - tic
ANSWERS.append(risk)

pass #print "Risk:, ", risk
pass #print "Sigma: ", Sigma.value

pass #print "Diagonal risk: ", x.T.dot(np.diag(np.diag(Sigma.value)).dot(x))


from huge_testman import *
import matplotlib.pyplot as plt
import numpy as np

per_change = []
errorbars = []

for fname in oldtime:
	os = oldtime[fname]
	if len(os) == 0:
		continue
	ns = newtime[fname]
	all_per_changes = [(o - n) / o for o , n in zip(os, ns)]
	a = np.array(all_per_changes)
	per_change.append(np.percentile(a , 50))
	errorbars.append( [ np.percentile(a, 25), np.percentile(a, 75) ] ) 

for idx, f in enumerate(fnames):
	s = len(f) - f[::-1].find('/')
	e = len(f) - f[::-1].find('.') - 1
	fnames[idx] = f[s:e]

zfname = zip(per_change, fnames)
zfname.sort(reverse=True)

print zfname
# per_change = [ z[0]  for z in zfname ]
# fnames = [ z[1]  for z in zfname ]
# fig, ax = plt.subplots()
# spaces = [1.5 * i + .2 for i in range(len(per_change)) ]
# rects = ax.bar(spaces, per_change, .45, color = 'y', yerr=tuple(errorbars))

# def autolabel(rects, labels, heights):
# 	 for label, rect, height in zip(labels, rects, heights):
# 	 	 ax.text(rect.get_x()+rect.get_width()/2., max(1.07*height,0), label,\
# 	 	 	ha='center', va='bottom')



# autolabel(rects,fnames, per_change)
# ax.set_xticklabels( tuple( ['' for i in range(len(fnames))]))
# plt.ylabel('Percent speed up')
# plt.show()




import numpy as np
from cvxpy import *
import time

ANSWERS = []
TIME = 0
m = 4 # the number of raw materials
n = 2 # the number of blended materials
q = 3 # the number of constituents
# the ith column of C is c_i
C = np.array([[ .9,  .8, .7, .6],
              [.08, .12, .2, .2],
              [.02, .08, .1, .2]])

# bounds on the blended product concentration
c_min = np.array([[.85, 0.65],
                    [  0,  0],
                    [  0,  0]])
c_max = np.array([[ 1,  1 ],
                  [.1, .18],
                  [.05, .17]])

FTilde = np.array([10, 10]) # limit on the flow rate of the blended material
F = np.array([7, 2, 6, 3]) # availibility of raw materials

p = np.array([15, 13, 11, 8]) # price of raw materials
pTilde = np.array([21, 18]) # price of the blended material


f_tilde = Variable(n)
f = Variable(m,n)
M = Variable(n,q)

constraints = [f >= 0, f_tilde <= FTilde]
for i in range(m):
	constraints.append(sum(f[i,:]) <= F[i])

for i in range(n):
	to_add = [f[k,i] * C[:,k] for k in range(m)]
	constraints.append(M[i,:].T == sum(to_add))
	constraints.append(c_min[:,i]*f_tilde[i]  <= M[i,:].T )
	constraints.append(c_max[:,i]*f_tilde[i]  >= M[i,:].T )
	constraints.append(sum( M[i,:]) == f_tilde[i] )

to_add = [-f[j,i] * p[j] for i in range(n) for j in range(m)]
obj = sum(to_add)
obj += sum([f_tilde[i] * pTilde[i] for i in range(n)])

objective = Maximize(obj)
problem = Problem(objective, constraints)
tic = time.time()
val = problem.solve()
toc = time.time()
TIME += toc - tic
ANSWERS.append(val)
pass #print val
pass #print f.value
pass #print f_tilde.value
for i in range(n):
	pass #print M.value[i,:] / f_tilde[i].value


from cvxpy import *
import numpy

x = Variable(2)

constraints = [ 2 * x[0] + x[1] >= 1, x[0] + 3 * x[1] >= 1, \
x[0] >= 0, x[1] >= 0 ]

objectives  =   [x[0] + x[1]]    

for n, obj in enumerate(objectives):
	prob = Problem(Minimize(obj), constraints)
	prob.solve()
	print "Solution to objective " + str(n)
	print x.value
	print obj.value
	print


# data for optimal evacuation problem
from cvxpy import *
import numpy as np
import matplotlib.pyplot as plt
import math
import time

ANSWERS = []
TIME = 0
T = 20
A = np.array([[-1.,-1., 0., 0., 0., 0., 0., 0., 0.],
 [ 1., 0.,-1., 0., 0., 0., 0., 0., 0.],
 [ 0., 0., 1.,-1., 0., 0., 0., 0., 0.],
 [ 0., 1., 0., 0.,-1.,-1., 0., 0., 0.],
 [ 0., 0., 0., 0., 1., 0.,-1., 0., 0.],
 [ 0., 0., 0., 1., 0., 0., 1.,-1., 0.],
 [ 0., 0., 0., 0., 0., 1., 0., 0.,-1.],
 [ 0., 0., 0., 0., 0., 0., 0., 1., 1.]])
settings.USE_CVXCANON = True
Q = np.array([ 1. , 1. , 1. , 1. , 1. , 0.8, 1. , 0.4])
F = np.array([ 0.25, 0.25, 0.25, 0.25, 0.25, 0.25, 0.25, 0.25, 0.25])
q1 = np.array([ 1., 0., 0., 0., 0., 0., 0., 0.])
r = np.asmatrix( np.array([ 1. , 0.2, 0.2, 0.5, 0.5, 0. , 0.5, 0. ]))
s = np.array([ 1. , 0.2, 0.2, 0.5, 0.5, 0. , 0.5, 0. ])
rtild = np.array([ 0.1, 0.2, 0.1, 5. , 0.4, 0.2, 0.4, 0.4, 0.2])
stild = np.array([   2.8,   5.6,   2.8, 140. ,  11.2,   5.6,  11.2,  11.2,   5.6])
s = np.asmatrix(s)
rtild = np.asmatrix(rtild)
stild = np.asmatrix(stild)


qs = [Variable(len(Q)) for _ in range(T)]
fs = [Variable(len(F)) for _ in range(T)]

obj_func = r * qs[0] + s * square(qs[0])  + rtild * abs(fs[0]) + stild * square(fs[0])

for i in range(1,T-1):
	obj_func += r * qs[i] + s * square(qs[i])  + rtild * abs(fs[i]) + stild * square(fs[i])

obj_func +=  r * qs[T-1] + s * square(qs[T-1])

constraints = [ qs[0] == q1 ]
for i in range(T):
	constraints.append(abs(fs[i]) <= F )
	constraints.append( qs[i] <= Q)
	constraints.append(qs[i] >= 0)

for i in range(T-1):
	constraints.append(qs[i+1] == A*fs[i] + qs[i])

obj = Minimize(obj_func)
prob = Problem(obj, constraints)

tic = time.time()
val = prob.solve(solver=ECOS)
toc = time.time()
TIME += toc - tic
ANSWERS.append(val)

# Plotting nodes
pass #plt.figure(1)
for i in range(len(Q)):
	pass #plt.plot([qs[t].value[i][0,0] for t in range(T) ])
	pass #plt.hold(True)
pass #plt.legend(["Node " + str(i+1) for i in range(1 + len(Q))])
pass #plt.title("Node occupancy during evacuation")
pass #plt.xlabel("Time")
pass #plt.ylabel("Node occupancy")
pass #plt.show()



pass #plt.figure(2)
for i in range(len(F)):
	pass #plt.plot([fs[t].value[i][0,0] for t in range(T) ])
	pass #plt.hold(True)
pass #plt.legend(["Edge " + str(i+1) for i in range(1 + len(F))])
pass #plt.title("Edge flow during evacuation")
pass #plt.xlabel("Time")
pass #plt.ylabel("Edge flow")
pass #plt.show()



pass #plt.figure(3)

exposures = [r * np.asmatrix(qs[i].value ) + s * np.square(np.asmatrix(qs[i].value ))\
+ rtild * np.abs(np.asmatrix(fs[i].value )) + stild * np.square(np.asmatrix(fs[i].value ) ) for i in range(T)]

exposures = [ exposures[t][0,0] for t in range(T) ]
pass #plt.plot(   exposures ) 
pass #plt.title("Risk exposure flow during evacuation")
pass #plt.xlabel("Time")
pass #plt.ylabel("Exposure")
pass #plt.show()



import numpy as np
from cvxpy import *
import matplotlib.pyplot as plt
import time

ANSWERS = []
TIME = 0

A = np.zeros( (2,2) )
A[0][0] = 1
A[1][1] = 2
A[0][1] = A[1][0] = .5

x = Variable(2)
obj = Minimize( quad_form(x, A) - x[0])
u_1 = Parameter()
u_2 = Parameter()

constraints = [x[0] + 2*x[1] <= u_1,\
x[0] - 4 * x[1] <= u_2,\
5 * x[0] + 76 * x[1] <= 1]

u_1.value = -2
u_2.value = -3

prob = Problem(obj, constraints)

tic = time.time()
p_star = prob.solve()
toc = time.time()
TIME += toc - tic
ANSWERS.append(p_star)
print "Optimal value,", p_star

print "x,", x.value

print "Lambda 1", constraints[0].dual_value
print "Lambda 2", constraints[1].dual_value
print "Lambda 3", constraints[2].dual_value

delta_1 = [0, -.1, .1]
delta_2 = [0, -.1, .1]

for d1 in delta_1:
	for d2 in delta_2:
		u_1.value = -2 + d1 
		u_2.value = -3 + d2
		val = prob.solve()
		ANSWERS.append(val)
		print d1, d2, val



import numpy as np
from cvxpy import *
import copy
import time

# data for power flow problem
import numpy as np
n = 12     # total number of nodes
m = 18     # number of edges (transmission lines)
k = 4      # number of generators
# transmission line capacities = 
TIME = 0

Pmax = np.matrix("""
    4.8005,
    1.9246,
    3.4274,
    2.9439,
    4.5652,
    4.0484,
    2.8259,
    1.0740,
    4.2856,
    2.7788,
    3.4617,
    4.1677,
    4.6873,
    3.9528,
    1.7051,
    2.6228,
    4.7419,
    4.6676,
""")
Gmax = np.matrix("3; 2; 4; 7")  # maximum generator power
c    = np.matrix("4; 8; 5; 3")  # supply generator costs
d = np.matrix("""                 
    1.6154
    2.3405
    1.0868
    1.5293
    2.2197
    1.0148
    1.2083
    1.3041
""")# network power demands
# graph incidence matrix
A = np.matrix(""" -1 -1  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0 ;
       0  0 -1 -1  0  0  0  0  0  0  0  0  0  0  0  0  0  0 ;
       0  0  0  0  0  0  0  0  0 -1 -1  0  0  0  0  0  0 -1 ;
       0  0  0  0  0  0 -1  0  0  0  0  0  0  0 -1  0 -1  0 ;
       1  0  0  0  1 -1  0  0  0  0  0  0  0  0  0  0  0  0 ;
       0  1  1  0 -1  0  1 -1  0  0  0  0  0  0  0  0  0  0 ;
       0  0  0  1  0  0  0  0 -1  1  0  0  0  0  0  0  0  0 ;
       0  0  0  0  0  0  0  1  1  0  0  0 -1  0  1  0  0  1 ;
       0  0  0  0  0  0  0  0  0  0  1 -1  0  0  0  0  0  0 ;
       0  0  0  0  0  0  0  0  0  0  0  1  1 -1  0  0  0  0 ;
       0  0  0  0  0  0  0  0  0  0  0  0  0  1  0  1  0  0 ;
       0  0  0  0  0  1  0  0  0  0  0  0  0  0  0 -1  1  0 
       """)


g = Variable(k)
p = Variable(m)
obj = Minimize(c.T*g)
constraints = [A*p == vstack(-g, d.T), abs(p) <= Pmax.T, 0 <= g, g <= Gmax]
prob = Problem(obj, constraints)
tic = time.time()
val = prob.solve()
toc = time.time()
TIME += toc - tic
ANSWERS.append(val)

pass #print val
pass #print g.value




# N - 1 contingency
g = Variable(k)
flows = []

obj = Minimize(c.T*g)

for i in range(m):
	flows.append(Variable(m))
constraints = [g <= Gmax, 0 <= g]

for i in range(m): # N -1 redundancy
	constraints.append(A*flows[i] == vstack(-g, d.T))
	constraints.append( flows[i][i] == 0 )
	constraints.append( abs(flows[i]) <= Pmax.T )

prob = Problem(obj, constraints)
tic = time.time()
val = prob.solve()
toc = time.time()
TIME += toc - tic
ANSWERS.append(val)

pass #print val
pass #print g.value


# the code below is not data for the problem
# it is used only to generate the network graph

# x-y coordinates
# each row is node x-y coordinates
XY = np.matrix("""     
 1.5 5.2;   
 4.9 5;     
 6.9 3.5;   
 1.9 3.5;   
 0.2 4.4;   
 3.2 4.8;   
 5.9 4.5;   
 3.9 3.6;   
 5.9 2.5;   
 3.9 3;     
 1.4 2.5;   
 0 3
 """);    

# node adjacency matrix
Ad = -A*A.T 
Ad = Ad - np.diag(np.diag(Ad))
epsx = 0.05; epsy = 0.15; # text placing offset

# plotting
import matplotlib.pyplot as plt
for i in range(12): #plot edges
    for j in range(i):
        if Ad[i,j] == 1:
            pass #plt.plot((XY[i,0], XY[j,0]), (XY[i,1], XY[j,1]), 'k-')
for j in range(k): #plot nodes
    pass #plt.plot(XY[j,0],XY[j,1],'rs', markersize=12)
    pass #plt.text(XY[j,0]-epsx,XY[j,1]+epsy,str(j+1))      
for j in range(k,n):
    pass #plt.plot(XY[j,0],XY[j,1],'ko')
pass #plt.axis('off')
pass #plt.savefig('pwr_net.eps')
if __name__ == '__main__':
    pass #plt.show()


from cvxpy import *
import numpy as np
import time


# Create two scalar optimization variables.

ANSWERS = []
TIME = 0

A = np.array([ [1, 2, 0, 1], \
[0, 0, 3, 1], \
[0, 3, 1, 1], \
[2, 1, 2, 5], \
[1, 0, 3, 2] ])

A_star = hstack(A,A)

c_max = np.array([100] * 5)

p = np.array([3, 2, 7, 6])
p_disc = np.array([2, 1, 4, 2])

p_star = vstack(p, p_disc)

q = np.array([4, 10, 5, 10])

x_star = Variable( 8 )
constraints = [ A_star * x_star <= c_max, x_star >= 0 ]
for i in range(4):
	constraints.append( x_star[i] >= q[i] )

objective = Maximize(p_star.T * x_star)

prob = Problem(objective, constraints)
tic = time.time()
ANSWERS.append( prob.solve() ) 
toc = time.time()
TIME += toc - tic 


x = np.array( [0] * 4)
for i in range(4):
	x[i] = x_star.value[i] + x_star.value[4 + i]


pass #print "Optimal revenue:", result 
pass #print "Optimal activity levels:", x

average_rate = np.array([0] * 4)

for i in range(4):
	average_rate[i] = (x_star.value[i] * p_star.value[i] + x_star.value[i + 4] * p_star.value[i + 4]) / x[i]

pass #print "Average rate:", average_rate


import numpy as np
from cvxpy import *
import math


ANSWERS = []
A = np.array(np.mat('-1 -1 0 0 0;\
	0 1 -1 -1 0;\
	0 0 0 1 1'))

I = Variable(5)
I_entr = Variable(2)
f_0 = -entr(I_entr[0])-I[2] - 26.0 * log(I[2])
f_0 += -entr(I_entr[1])-I[3]  - 26.0 * log(I[3])
constraints = []
constraints = [I_entr[0] == I[2] - 26.0, I_entr[1] == I[3] - 26.0 ]
constraints.append(A*I == 0)
objective = Minimize(f_0)
prob = Problem(objective, constraints)
val = prob.solve()

ANSWERS.append(val)
v = [None] * 5
v[0] = 1000*I.value[0]+I.value[0]
v[1] = 1000*I.value[1] 
v[4] = 100*I.value[4] 
v[3] =    math.log(1+ I.value[3]/26.0) 
v[2] =    math.log(1 + I.value[2]/26.0) 

v = np.mat(v)
e = np.linalg.pinv(A.T)*v.T

pass #print e == A * v.T

pass #print e
pass #print v


import numpy as np
from cvxpy import *
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm
import time

TIME = 0

h = 1.
g = 0.1
m = 10.
Fmax = 10.
p0 = np.matrix('50 ;50; 100')
v0 = np.matrix('-10; 0; -10')
alpha = 0.5
gamma = 1.
K = 35
ANSWERS = []


ps = []
val = 0
while val != float('inf'):  
	pass #print "K=", K
	v =  Variable( 3, K) 
	p =  Variable( 3, K)  
	f =  Variable( 3, K) 


	# Minimizing fuel
	obj_sum = norm( f[:,0] )
	for i in range(1,K):
		obj_sum += norm( f[:,i] )

	obj = Minimize(obj_sum)
	constraints = [v[:,0] == v0, p[:,0] == p0, p[:,K-1] == 0, v[:,K-1] == 0]


	for i in range(K):
		constraints.append( norm( f[:,i] ) <= Fmax )


	for i in range(1, K):
		constraints.append(  v[:,i] == v[:, i - 1] + (h/m) * f[:, i - 1] - h * g * np.array([0, 0, 1])  )
		constraints.append( p[:,i] == p[:,i - 1] + h * (h/2) * (v[:,i] + v[:,i - 1])  )

	for i in range(K):
		constraints.append( p[2,i] >= alpha * norm( p[1:,i])  )

	
	prob = Problem(obj, constraints)
	ps.append(p)
	tic = time.time()
	val = prob.solve()
	toc = time.time()
	TIME += toc - tic
	ANSWERS.append(val)
	K -= 1

pass #print toc - tic


p = ps[-2]# Last p which was feasible


# use the following code to plot your trajectories
# and the glide cone (don't modify)
# -------------------------------------------------------
# fig = pass #plt.figure()
# ax = fig.gca(projection='3d')

X = np.linspace(-40, 55, num=30)
Y = np.linspace(0, 55, num=30)
X, Y = np.meshgrid(X, Y)
Z = alpha*np.sqrt(X**2+Y**2)
# ax.plot_surface(X, Y, Z, rstride=1, cstride=1, cmap=cm.coolwarm, linewidth=0)
#Have your solution be stored in p

# ax.plot(xs=p.value[0,:].A1,ys=p.value[1,:].A1,zs=p.value[2,:].A1)
# ax.set_xlabel('x'); ax.set_ylabel('y'); ax.set_zlabel('z')


pass #plt.title('Minimum time path ')
pass #plt.show()


#    This file is part of CVXcanon.
#
#    CVXcanon is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    CVXcanon is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with CVXcanon.  If not, see <http:#www.gnu.org/licenses/>.

from . import CVXcanon
import numpy as np
from cvxpy.lin_ops.lin_op import *
import scipy.sparse
from collections import deque

def get_problem_matrix(constrs, id_to_col=None, constr_offsets=None):
    '''
    Builds a sparse representation of the problem data by calling CVXCanon's
    C++ build_matrix function.

    Parameters
    ----------
        constrs: A list of python linOp trees
        id_to_col: A map from variable id to offset withoun our matrix

    Returns
    ----------
        V, I, J: numpy arrays encoding a sparse representation of our problem
        const_vec: a numpy column vector representing the constant_data in our problem
    '''
    linOps = [constr.expr for constr in constrs]
    lin_vec = CVXcanon.LinOpVector()

    id_to_col_C = CVXcanon.IntIntMap()
    if id_to_col is None:
        id_to_col = {}

    # Loading the variable offsets from our
    # Python map into a C++ map
    for id, col in id_to_col.items():
        id_to_col_C[id] = col

    # This array keeps variables data in scope
    # after build_lin_op_tree returns
    tmp = []
    for lin in linOps:
        tree = build_lin_op_tree(lin, tmp)
        tmp.append(tree)
        lin_vec.push_back(tree)

    if constr_offsets is None:
        problemData = CVXcanon.build_matrix(lin_vec, id_to_col_C)
    else:
        # Load constraint offsets into a C++ vector
        constr_offsets_C = CVXcanon.IntVector()
        for offset in constr_offsets:
            constr_offsets_C.push_back(int(offset))
        problemData = CVXcanon.build_matrix(lin_vec, id_to_col_C,
                                            constr_offsets_C)

    # Unpacking
    V = problemData.getV(len(problemData.V))
    I = problemData.getI(len(problemData.I))
    J = problemData.getJ(len(problemData.J))
    const_vec = problemData.getConstVec(len(problemData.const_vec))

    return V, I, J, const_vec.reshape(-1, 1)


def format_matrix(matrix, format='dense'):
    ''' Returns the matrix in the appropriate form,
        so that it can be efficiently loaded with our swig wrapper
    '''
    if (format == 'dense'):
        # Ensure is 2D.
        matrix = np.atleast_2d(matrix)
        return np.asfortranarray(matrix)
    elif(format == 'sparse'):
        return scipy.sparse.coo_matrix(matrix)
    elif(format == 'scalar'):
        return np.asfortranarray(np.matrix(matrix))
    else:
        raise NotImplementedError()


def set_matrix_data(linC, linPy):
    '''  Calls the appropriate CVXCanon function to set the matrix data field of our C++ linOp.
    '''
    if isinstance(linPy.data, LinOp):
        if linPy.data.type is 'sparse_const':
            coo = format_matrix(linPy.data.data, 'sparse')
            linC.set_sparse_data(coo.data, coo.row.astype(float),
                                 coo.col.astype(float), coo.shape[0], coo.shape[1])
        elif linPy.data.type is 'dense_const':
            linC.set_dense_data(format_matrix(linPy.data.data))
        else:
            raise NotImplementedError()
    else:
        if linPy.type is 'sparse_const':
            coo = format_matrix(linPy.data, 'sparse')
            linC.set_sparse_data(coo.data, coo.row.astype(float),
                                 coo.col.astype(float), coo.shape[0], coo.shape[1])
        else:
            linC.set_dense_data(format_matrix(linPy.data))


def set_slice_data(linC, linPy):
    '''
    Loads the slice data, start, stop, and step into our C++ linOp.
    The semantics of the slice operator is treated exactly the same as in Python.
    Note that the 'None' cases had to be handled at the wrapper level, since we must load
    integers into our vector.
    '''
    for i, sl in enumerate(linPy.data):
        vec = CVXcanon.IntVector()

        start = 0
        if sl.start is not None:
            start = sl.start

        stop = linPy.args[0].size[i]
        if sl.stop is not None:
            stop = sl.stop

        step = 1
        if sl.step is not None:
            step = sl.step

        # handle [::-1] case
        if step < 0 and sl.start is None and sl.stop is None:
            tmp = start
            start = stop - 1
            stop = tmp

        for var in [start, stop, step]:
            vec.push_back(var)

        linC.slice.push_back(vec)


type_map = {
    "VARIABLE": CVXcanon.VARIABLE,
    "PROMOTE": CVXcanon.PROMOTE,
    "MUL": CVXcanon.MUL,
    "RMUL": CVXcanon.RMUL,
    "MUL_ELEM": CVXcanon.MUL_ELEM,
    "DIV": CVXcanon.DIV,
    "SUM": CVXcanon.SUM,
    "NEG": CVXcanon.NEG,
    "INDEX": CVXcanon.INDEX,
    "TRANSPOSE": CVXcanon.TRANSPOSE,
    "SUM_ENTRIES": CVXcanon.SUM_ENTRIES,
    "TRACE": CVXcanon.TRACE,
    "RESHAPE": CVXcanon.RESHAPE,
    "DIAG_VEC": CVXcanon.DIAG_VEC,
    "DIAG_MAT": CVXcanon.DIAG_MAT,
    "UPPER_TRI": CVXcanon.UPPER_TRI,
    "CONV": CVXcanon.CONV,
    "HSTACK": CVXcanon.HSTACK,
    "VSTACK": CVXcanon.VSTACK,
    "SCALAR_CONST": CVXcanon.SCALAR_CONST,
    "DENSE_CONST": CVXcanon.DENSE_CONST,
    "SPARSE_CONST": CVXcanon.SPARSE_CONST,
    "NO_OP": CVXcanon.NO_OP,
    "KRON": CVXcanon.KRON
}


def get_type(ty):
    if ty in type_map:
        return type_map[ty]
    else:
        raise NotImplementedError()


def build_lin_op_tree(root_linPy, tmp):
    '''
    Breadth-first, pre-order traversal on the Python linOp tree
    Parameters
    -------------
    root_linPy: a Python LinOp tree

    tmp: an array to keep data from going out of scope

    Returns
    --------
    root_linC: a C++ LinOp tree created through our swig interface
    '''
    Q = deque()
    root_linC = CVXcanon.LinOp()
    Q.append((root_linPy, root_linC))

    while len(Q) > 0:
        linPy, linC = Q.popleft()

        # Updating the arguments our LinOp
        for argPy in linPy.args:
            tree = CVXcanon.LinOp()
            tmp.append(tree)
            Q.append((argPy, tree))
            linC.args.push_back(tree)

        # Setting the type of our lin op
        linC.type = get_type(linPy.type.upper())

        # Setting size
        linC.size.push_back(int(linPy.size[0]))
        linC.size.push_back(int(linPy.size[1]))

        # Loading the problem data into the appropriate array format
        if linPy.data is None:
            pass
        elif isinstance(linPy.data, tuple) and isinstance(linPy.data[0], slice):
            set_slice_data(linC, linPy)
        elif isinstance(linPy.data, float) or isinstance(linPy.data, int):
            linC.set_dense_data(format_matrix(linPy.data, 'scalar'))
        elif isinstance(linPy.data, LinOp) and linPy.data.type is 'scalar_const':
            linC.set_dense_data(format_matrix(linPy.data.data, 'scalar'))
        else:
            set_matrix_data(linC, linPy)

    return root_linC



from cvxpy import *
import numpy
import time

ANSWERS = []
TIME = 0 
x = Variable(2)

constraints = [ 2 * x[0] + x[1] >= 1, x[0] + 3 * x[1] >= 1, \
x[0] >= 0, x[1] >= 0 ]

objectives  =   [x[0] + x[1], -x[0] - x[1], x[0],\
max_elemwise( x[0], x[1] ), x[0] ** 2 + 9 * x[1] ** 2 ]    

for n, obj in enumerate(objectives):
	prob = Problem(Minimize(obj), constraints)
	tic = time.time()
	ANSWERS.append(prob.solve())
	toc = time.time()
	TIME += toc - tic
	pass #print "Solution to objective " + str(n)
	pass #print x.value
	pass #print obj.value
	pass #print


import numpy as np
from cvxpy import *
import matplotlib.pyplot as plt
import time

np.random.seed(1)
N = 100
ANSWERS = []
TIME = 0

# create an increasing input signal
xtrue = np.zeros(N)
xtrue[0:40] = 0.1
xtrue[49] = 2
xtrue[69:80] = 0.15
xtrue[79] = 1
xtrue = np.cumsum(xtrue)

# pass the increasing input through a moving-average filter 
# and add Gaussian noise
h = np.array([1, -0.85, 0.7, -0.3])
k = h.size
yhat = np.convolve(h,xtrue)
y = yhat[0:-3].reshape(N,1) + np.random.randn(N,1)

xtrue = np.asmatrix(xtrue.reshape(N,1))
y = np.asmatrix(y)

v = Variable(N)
x = Variable(N)

constraints = [x >= 0]
for i in range(N - 1):
	constraints.append( x[i+1] >= x[i] )

constraints.append( y == ( conv(h,x)[0:-3] + v ) )

obj = Minimize( norm(v) )
prob = Problem(obj, constraints)
tic = time.time()
ANSWERS.append(prob.solve())
toc = time.time()
TIME += toc - tic

pass #print x.value
pass #print v.value

pass #plt.figure(1)
pass #plt.plot(xtrue)
pass #plt.plot(x.value)
pass #plt.legend(["True signal", "MLE signal"])
pass #plt.title("Maximum likelihood reconstruction of FIR filter with constraints")
pass #plt.show()



constraints.append( y == ( conv(h,x)[0:-3] + v ) )
obj = Minimize( norm(v) )
prob = Problem(obj, constraints)
ANSWERS.append(prob.solve())

pass #plt.figure(1)
pass #plt.plot(xtrue)
pass #plt.plot(x.value)
pass #plt.legend(["True signal", "MLE signal"])
pass #plt.title("Maximum likelihood reconstruction of FIR filter no constraints")
pass #plt.show()


import numpy as np
from cvxpy import *
import matplotlib.pyplot as plt
import copy



from scipy import linalg as la
np.random.seed(8)

n = 5 # Dimension
N = 100 # Number of sample
N_test = 10 # Samples for test set

X = np.random.randn(n,N)
Y = np.random.randn(n,N)

X_test = np.random.randn(n,N_test)
Y_test = np.random.randn(n,N_test)

P = np.random.randn(n,n)
P = P.dot(P.T) + np.identity(n)
sqrtP = la.sqrtm(P)

d = np.linalg.norm(sqrtP.dot(X-Y),axis=0)
d = np.maximum(d+np.random.randn(N),0)

d_test = np.linalg.norm(sqrtP.dot(X_test-Y_test),axis=0)
d_test = np.maximum(d_test+np.random.randn(N_test),0)



Z = X - Y  


P = Semidef(n)
objective = quad_form( Z[:,0] ,P )**2  + d[0] **2 -  2*d[0]* sqrt(quad_form(Z[:,0],P)  )
for i in range(1,N):
	objective = quad_form(Z[:,i],P )**2  + d[i] **2 -2*d[i]* sqrt(quad_form(Z[:,i],P) )

#objective /= float(N)

obj = Minimize(objective)
prob = Problem(obj, [])
val = prob.solve()

pass #print "P", P.value
pass #print "training error", val


testing_error = 0

Z_test = X_test - Y_test
for i in range(N_test):
	testing_error += (d_test[i] -  Z_test[:,i].T.dot( P.value.dot(Z_test[:,i]).T))**2

pass #print "Testing error:", testing_error


import numpy as np
from cvxpy import *
import matplotlib.pyplot as pyplot
import heapq
import time

settings.USE_CVXCANON = True
ANSWERS = []
TIME = 0
np.random.seed(0)
m=100
k=40 # max # permuted measurements
n=20
A=10 * np.random.randn(m,n)
x_true=np.random.randn(n,1) # true x value
y_true = A.dot(x_true) + np.random.randn(m,1)

# build permuted indices
perm_idxs=np.random.permutation(m)
perm_idxs=np.sort(perm_idxs[:k])
temp_perm=np.random.permutation(k)
new_pos=np.zeros(k)
for i in range(k):
  new_pos[i] = perm_idxs[temp_perm[i]]
new_pos = new_pos.astype(int)

# true permutation matrix
P=np.identity(m)
P[perm_idxs,:]=P[new_pos,:]
true_perm=[]

for i in range(k):
  if perm_idxs[i] != new_pos[i]:
    true_perm = np.append(true_perm, perm_idxs[i])

y = P.dot(y_true)
new_pos = None



P_fixed = np.identity(m)
x_fixed = None


def optimizeP(A, x, y):
	P = np.identity(m)
	Ax = A.dot(x)
	
	Ax_largest = heapq.nlargest(m, range(len(Ax)), Ax.take)
	y_largest = heapq.nlargest(m, range(len(y)), y.take)


	for i in range(m):
		P[ y_largest[i], y_largest[i] ] = 0
		P[ y_largest[i], Ax_largest[i] ] = 1

	return P



def numPermuted(P):
	result = 0
	for i in range(m):
		if P[i,i] != 1:
			result += 1
	return result		

firstIter = None

for _ in range(20):

	x = Variable(n)
	objective = Minimize( norm( A*x - P_fixed.T.dot( y)) )
	constraints = []
	prob = Problem(objective, constraints)
	tic = time.time()
	result = prob.solve()
	toc = time.time()
	TIME += toc - tic
	ANSWERS.append(result)
	if firstIter is None:
		firstIter = result
	x_fixed = x.value

	P_fixed = optimizeP(A, x_fixed, y)


print "Num permuted the same as k: ",  numPermuted(P) == k 
print "Final objective", result
print "P = I error", firstIter



from cvxpy import *
import numpy as np

# x = Variable(2)
# x = Variable(2)
# y = Variable(2)
# print Problem(Minimize(0), [x + y <= [1, 0]]).solve()

x = Variable(2)
constraints = [ 2 * x[0] + x[1] >= 1, x[0] + 3 * x[1] >= 1, \
x[0] >= 0, x[1] >= 0 ]

objectives  =   [x[0] + x[1], -x[0] - x[1], x[0],\
max_elemwise( x[0], x[1] ), x[0] ** 2 + 9 * x[1] ** 2 ]

for n, obj in enumerate(objectives):
	settings.USE_CVXCANON = False

	print "\n\nNormal Solution:\n"
	prob = Problem(Minimize(obj), constraints)
	prob.solve()
	print "Solution to objective " + str(n)
	print x.value
	print obj.value

	settings.USE_CVXCANON = True
	
	print "\n\nCVXCANON Solution:\n"
	prob = Problem(Minimize(obj), constraints)
	prob.solve()
	print "Solution to objective " + str(n)
	print x.value
	print obj.value


import numpy as np
from cvxpy import *
import matplotlib.pyplot as plt
import time

ANSWERS = []
TIME = 0
np.random.seed(0)
(m, n) = (300, 100)
A = np.random.rand(m, n); A = np.asmatrix(A)
b = A.dot(np.ones((n, 1)))/2; b = np.asmatrix(b)
c = - np.random.rand(n, 1); c = np.asmatrix(c)
# Solving the LP relaxation 
x = Variable(n)
ts = np.arange(0,1,.01)
obj = Minimize(c.T * x)

constraints = [A*x <= b, 0 <= x, x <= 1]

prob = Problem(obj, constraints)
tic = time.time()
L = prob.solve()
toc = time.time()
TIME += toc - tic
ANSWERS.append(L)


vals = []
maximum_failure = []
feasible_vals = []
feasible_ts = []
feasible_t = []

for t in ts:
	x_hat = np.zeros( (n,1) )
	for i in range(n):
		if(x.value[i] >= t):
			x_hat[i] = 1.0

	vals.append(c.T * x_hat)
	
	if np.all(A * x_hat <= b):
		feasible_t.append(True)
		feasible_vals.append(c.T * x_hat)
		feasible_ts.append(t)
	else:
		feasible_t.append(False)

	maximum_failure.append(np.amax(A*x_hat - b))

min_t = np.argmin(feasible_vals)
pass #print "Best feasible t, ", feasible_ts[min_t]

for i in range(len(vals)):
	if(feasible_t[i]):
		pass #plt.scatter(ts[i], vals[i], marker='o')
	else:
		pass #plt.scatter(ts[i], vals[i], marker='x')

pass #plt.xlabel('t')
pass #plt.ylabel('Objective value')
pass #plt.title('LP Relaxation')
pass #plt.show()

pass #print maximum_failure

pass #plt.plot(ts, maximum_failure)
pass #plt.xlabel('t')
pass #plt.ylabel('Maximum failure')
pass #plt.show()



from cvxpy import *
import numpy
import time


ANSWERS = []
TIME = 0
# SVM with indexing.
def gen_data(n):
    pos = numpy.random.multivariate_normal([1.0,2.0],numpy.eye(2),size=n)
    neg = numpy.random.multivariate_normal([-1.0,1.0],numpy.eye(2),size=n)
    return pos, neg

N = 2
C = 10
pos, neg = gen_data(500)

w = Variable(N)
b = Variable()
xi_pos = Variable(pos.shape[0])
xi_neg = Variable(neg.shape[0])
cost = sum_squares(w) + C*sum_entries(xi_pos) + C*sum_entries(xi_neg)
constrs = []
for j in range(pos.shape[0]):
    constrs += [w.T*pos[j,:] - b >= 1 - xi_pos[j]]
    
for j in range(neg.shape[0]):
    constrs += [-(w.T*neg[j,:] - b) >= 1 - xi_neg[j]]

p = Problem(Minimize(cost), constrs)

tic =  time.time()
ANSWERS.append(p.solve())
toc = time.time()
TIME += toc - tic


from setuptools import setup, Extension, find_packages
from setuptools.command.install import install
from distutils.command.build import build
import numpy

canon = Extension(
    'CVXcanon._CVXcanon',
    sources=['src/CVXcanon.cpp', 'src/LinOpOperations.cpp', 'CVXcanon/CVXcanon_wrap.cpp'],
    # swig_opts=['-c++', '-outdir', 'CVXcanon', '-I./src/', '-I./CVXcanon/'],
    include_dirs=['src/', 'CVXcanon/', 'include/Eigen', numpy.get_include()]
)


# Ensure the swig generated CVXcanon.py module is present before python
# copies sources to the build directory
class CustomBuild(build):
    def run(self):
        self.run_command('build_ext')
        build.run(self)


# Ensure the swig generated CVXcanon.py module is present before python
# copies sources to the build directory
class CustomInstall(install):
    def run(self):
        self.run_command('build_ext')
        self.do_egg_install()


setup(
    name='CVXcanon',
    version='0.0.20',
    setup_requires=['numpy'],
    author='Jack Zhu, John Miller, Paul Quigley',
    author_email='jackzhu@stanford.edu, millerjp@stanford.edu, piq93@stanford.edu',
    ext_modules=[canon],
    packages=find_packages(),
    py_modules=['canonInterface'],
    description='A low-level library to perform the matrix building step in cvxpy, a convex optimization modeling software.',
    license='GPLv3',
    url='https://github.com/jacklzhu/CVXcanon',
    cmdclass={'build': CustomBuild, 'install': CustomInstall},
    install_requires=[
        'numpy',
        'scipy',
        #'cvxpy', CIRCULAR DEPENDENCY!
    ]
)
